/*  Copyright (C) 2012-2024 by László Nagy
    This file is part of Bear.

    Bear is a tool to generate compilation database for clang tooling.

    Bear is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bear is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <new>

/**
 * To avoid undefined symbol from `libc++.so` or `libstdc++.so` define a
 * no-op delete operator. It's safe to not call `free` because we are not
 * `malloc` or `new` in this library.
 *
 * But it's needed since we were declaring a few classes virtual, which
 * requires this symbol to be present.
 */
void operator delete(void*, std::size_t)
{ }

void operator delete(void*)
{ }
