/*  Copyright (C) 2012-2024 by László Nagy
    This file is part of Bear.

    Bear is a tool to generate compilation database for clang tooling.

    Bear is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Bear is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "report/wrapper/EventReporter.h"

namespace wr {

    EventReporter::EventReporter(const SessionLocator &session_locator) noexcept
            : event_factory()
            , client(session_locator)
    { }

    void EventReporter::report_start(ProcessId pid, const Execution &execution) {
        const auto &event = event_factory.start(pid, getppid(), execution);
        client.report(event);
    }

    void EventReporter::report_wait(const sys::ExitStatus exit_status) {
        const auto event = (exit_status.is_signaled())
                ? event_factory.signal(exit_status.signal().value())
                : event_factory.terminate(exit_status.code().value());
        client.report(event);
    }
}
